"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _regenerator = require("babel-runtime/regenerator");

var _regenerator2 = _interopRequireDefault(_regenerator);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _util = require("./util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Kuroshiro Class
 */
var Kuroshiro = function () {
    /**
     * Constructor
     * @constructs Kuroshiro
     */
    function Kuroshiro() {
        _classCallCheck(this, Kuroshiro);

        this._analyzer = null;
    }

    /**
     * Initialize Kuroshiro
     * @memberOf Kuroshiro
     * @instance
     * @returns {Promise} Promise object represents the result of initialization
     */


    _createClass(Kuroshiro, [{
        key: "init",
        value: function () {
            var _ref = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(analyzer) {
                var self;
                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                self = this;

                                if (!(self._analyzer == null)) {
                                    _context.next = 13;
                                    break;
                                }

                                _context.prev = 2;
                                _context.next = 5;
                                return analyzer.init();

                            case 5:
                                self._analyzer = analyzer;
                                _context.next = 11;
                                break;

                            case 8:
                                _context.prev = 8;
                                _context.t0 = _context["catch"](2);
                                throw _context.t0;

                            case 11:
                                _context.next = 14;
                                break;

                            case 13:
                                throw new Error("Kuroshiro has already been initialized.");

                            case 14:
                            case "end":
                                return _context.stop();
                        }
                    }
                }, _callee, this, [[2, 8]]);
            }));

            function init(_x) {
                return _ref.apply(this, arguments);
            }

            return init;
        }()

        /**
         * Convert given string to target syllabary with options available
         * @memberOf Kuroshiro
         * @instance
         * @param {string} str Given String
         * @param {Object} [options] JSON object which have key-value pairs settings
         * @param {string} [options.to='hiragana'] Target syllabary ['hiragana'|'katakana'|'romaji']
         * @param {string} [options.mode='normal'] Convert mode ['normal'|'spaced'|'okurigana'|'furigana']
         * @param {string} [options.delimiter_start='('] Delimiter(Start)
         * @param {string} [options.delimiter_end=')'] Delimiter(End)
         * @returns {Promise} Promise object represents the result of conversion
         */

    }, {
        key: "convert",
        value: function () {
            var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2(str, options) {
                var tokens, cr, hi, tmp, hpattern, hc, hreg, hmatches, pickKJ, hc1, notations, i, strType, pattern, isLastTokenKanji, subs, c, reg, matches, pickKanji, c1, c2, c3, result, n0, n1, n2, n3, n4, n5;
                return _regenerator2.default.wrap(function _callee2$(_context2) {
                    while (1) {
                        switch (_context2.prev = _context2.next) {
                            case 0:
                                options = options || {};
                                options.to = options.to || "hiragana";
                                options.mode = options.mode || "normal";
                                // options.convertall = options.convertall || false;
                                options.delimiter_start = options.delimiter_start || "(";
                                options.delimiter_end = options.delimiter_end || ")";
                                str = str || "";

                                _context2.next = 8;
                                return this._analyzer.parse(str);

                            case 8:
                                tokens = _context2.sent;

                                for (cr = 0; cr < tokens.length; cr++) {
                                    if ((0, _util.hasJapanese)(tokens[cr].surface_form)) {
                                        if (!tokens[cr].reading) {
                                            if (tokens[cr].surface_form.split().every(_util.isKana)) {
                                                tokens[cr].reading = (0, _util.toRawKatakana)(tokens[cr].surface_form);
                                            } else {
                                                tokens[cr].reading = tokens[cr].surface_form;
                                            }
                                        } else if ((0, _util.hasHiragana)(tokens[cr].reading)) {
                                            tokens[cr].reading = (0, _util.toRawKatakana)(tokens[cr].reading);
                                        }
                                    } else {
                                        tokens[cr].reading = tokens[cr].surface_form;
                                    }
                                }

                                if (!(options.mode === "normal" || options.mode === "spaced")) {
                                    _context2.next = 27;
                                    break;
                                }

                                _context2.t0 = options.to;
                                _context2.next = _context2.t0 === "katakana" ? 14 : _context2.t0 === "romaji" ? 17 : _context2.t0 === "hiragana" ? 20 : 24;
                                break;

                            case 14:
                                if (!(options.mode === "normal")) {
                                    _context2.next = 16;
                                    break;
                                }

                                return _context2.abrupt("return", (0, _util.splitObjArray)(tokens, "reading"));

                            case 16:
                                return _context2.abrupt("return", (0, _util.splitObjArray)(tokens, "reading", " "));

                            case 17:
                                if (!(options.mode === "normal")) {
                                    _context2.next = 19;
                                    break;
                                }

                                return _context2.abrupt("return", (0, _util.toRawRomaji)((0, _util.splitObjArray)(tokens, "reading")));

                            case 19:
                                return _context2.abrupt("return", (0, _util.toRawRomaji)((0, _util.splitObjArray)(tokens, "reading", " ")));

                            case 20:
                                for (hi = 0; hi < tokens.length; hi++) {
                                    if ((0, _util.hasKanji)(tokens[hi].surface_form)) {
                                        if (!(0, _util.hasKatakana)(tokens[hi].surface_form)) {
                                            tokens[hi].reading = (0, _util.toRawHiragana)(tokens[hi].reading);
                                        } else {
                                            // handle katakana-kanji-mixed tokens
                                            tokens[hi].reading = (0, _util.toRawHiragana)(tokens[hi].reading);
                                            tmp = "";
                                            hpattern = "";

                                            for (hc = 0; hc < tokens[hi].surface_form.length; hc++) {
                                                if ((0, _util.isKanji)(tokens[hi].surface_form[hc])) {
                                                    hpattern += "(.*)";
                                                } else {
                                                    hpattern += (0, _util.isKatakana)(tokens[hi].surface_form[hc]) ? (0, _util.toRawHiragana)(tokens[hi].surface_form[hc]) : tokens[hi].surface_form[hc];
                                                }
                                            }
                                            hreg = new RegExp(hpattern);
                                            hmatches = hreg.exec(tokens[hi].reading);

                                            if (hmatches) {
                                                pickKJ = 0;

                                                for (hc1 = 0; hc1 < tokens[hi].surface_form.length; hc1++) {
                                                    if ((0, _util.isKanji)(tokens[hi].surface_form[hc1])) {
                                                        tmp += hmatches[pickKJ + 1];
                                                        pickKJ++;
                                                    } else {
                                                        tmp += tokens[hi].surface_form[hc1];
                                                    }
                                                }
                                                tokens[hi].reading = tmp;
                                            }
                                        }
                                    } else {
                                        tokens[hi].reading = tokens[hi].surface_form;
                                    }
                                }

                                if (!(options.mode === "normal")) {
                                    _context2.next = 23;
                                    break;
                                }

                                return _context2.abrupt("return", (0, _util.splitObjArray)(tokens, "reading"));

                            case 23:
                                return _context2.abrupt("return", (0, _util.splitObjArray)(tokens, "reading", " "));

                            case 24:
                                throw new Error("Unknown option.to param");

                            case 25:
                                _context2.next = 68;
                                break;

                            case 27:
                                if (!(options.mode === "okurigana" || options.mode === "furigana")) {
                                    _context2.next = 67;
                                    break;
                                }

                                notations = []; // [basic,basic_type[1=kanji,2=hiragana(katakana),3=others],notation]

                                i = 0;

                            case 30:
                                if (!(i < tokens.length)) {
                                    _context2.next = 54;
                                    break;
                                }

                                tokens[i].reading = (0, _util.toRawHiragana)(tokens[i].reading);

                                strType = (0, _util.getStrType)(tokens[i].surface_form);
                                _context2.t1 = strType;
                                _context2.next = _context2.t1 === 0 ? 36 : _context2.t1 === 1 ? 38 : _context2.t1 === 2 ? 46 : _context2.t1 === 3 ? 48 : 50;
                                break;

                            case 36:
                                notations.push([tokens[i].surface_form, 1, tokens[i].reading]);
                                return _context2.abrupt("break", 51);

                            case 38:
                                pattern = "";
                                isLastTokenKanji = false;
                                subs = []; // recognize kanjis and group them

                                for (c = 0; c < tokens[i].surface_form.length; c++) {
                                    if ((0, _util.isKanji)(tokens[i].surface_form[c])) {
                                        if (!isLastTokenKanji) {
                                            // ignore successive kanji tokens (#10)
                                            isLastTokenKanji = true;
                                            pattern += "(.*)";
                                            subs.push(tokens[i].surface_form[c]);
                                        } else {
                                            subs[subs.length - 1] += tokens[i].surface_form[c];
                                        }
                                    } else {
                                        isLastTokenKanji = false;
                                        subs.push(tokens[i].surface_form[c]);
                                        pattern += (0, _util.isKatakana)(tokens[i].surface_form[c]) ? (0, _util.toRawHiragana)(tokens[i].surface_form[c]) : tokens[i].surface_form[c];
                                    }
                                }
                                reg = new RegExp("^" + pattern + "$");
                                matches = reg.exec(tokens[i].reading);

                                if (matches) {
                                    pickKanji = 1;

                                    for (c1 = 0; c1 < subs.length; c1++) {
                                        if ((0, _util.isKanji)(subs[c1][0])) {
                                            notations.push([subs[c1], 1, matches[pickKanji++]]);
                                        } else {
                                            notations.push([subs[c1], 2, (0, _util.toRawHiragana)(subs[c1])]);
                                        }
                                    }
                                } else {
                                    notations.push([tokens[i].surface_form, 1, tokens[i].reading]);
                                }
                                return _context2.abrupt("break", 51);

                            case 46:
                                for (c2 = 0; c2 < tokens[i].surface_form.length; c2++) {
                                    notations.push([tokens[i].surface_form[c2], 2, tokens[i].reading[c2]]);
                                }
                                return _context2.abrupt("break", 51);

                            case 48:
                                for (c3 = 0; c3 < tokens[i].surface_form.length; c3++) {
                                    notations.push([tokens[i].surface_form[c3], 3, tokens[i].surface_form[c3]]);
                                }
                                return _context2.abrupt("break", 51);

                            case 50:
                                throw new Error("Unknown strType");

                            case 51:
                                i++;
                                _context2.next = 30;
                                break;

                            case 54:
                                result = "";
                                _context2.t2 = options.to;
                                _context2.next = _context2.t2 === "katakana" ? 58 : _context2.t2 === "romaji" ? 60 : _context2.t2 === "hiragana" ? 62 : 64;
                                break;

                            case 58:
                                if (options.mode === "okurigana") {
                                    for (n0 = 0; n0 < notations.length; n0++) {
                                        if (notations[n0][1] !== 1) {
                                            result += notations[n0][0];
                                        } else {
                                            result += notations[n0][0] + options.delimiter_start + (0, _util.toRawKatakana)(notations[n0][2]) + options.delimiter_end;
                                        }
                                    }
                                } else {
                                    // furigana
                                    for (n1 = 0; n1 < notations.length; n1++) {
                                        if (notations[n1][1] !== 1) {
                                            result += notations[n1][0];
                                        } else {
                                            result += "<ruby>" + notations[n1][0] + "<rp>" + options.delimiter_start + "</rp><rt>" + (0, _util.toRawKatakana)(notations[n1][2]) + "</rt><rp>" + options.delimiter_end + "</rp></ruby>";
                                        }
                                    }
                                }
                                return _context2.abrupt("return", result);

                            case 60:
                                if (options.mode === "okurigana") {
                                    for (n2 = 0; n2 < notations.length; n2++) {
                                        if (notations[n2][1] !== 1) {
                                            result += notations[n2][0];
                                        } else {
                                            result += notations[n2][0] + options.delimiter_start + (0, _util.toRawRomaji)(notations[n2][2]) + options.delimiter_end;
                                        }
                                    }
                                } else {
                                    // furigana
                                    result += "<ruby>";
                                    for (n3 = 0; n3 < notations.length; n3++) {
                                        result += notations[n3][0] + "<rp>" + options.delimiter_start + "</rp><rt>" + (0, _util.toRawRomaji)(notations[n3][2]) + "</rt><rp>" + options.delimiter_end + "</rp>";
                                    }
                                    result += "</ruby>";
                                }
                                return _context2.abrupt("return", result);

                            case 62:
                                if (options.mode === "okurigana") {
                                    for (n4 = 0; n4 < notations.length; n4++) {
                                        if (notations[n4][1] !== 1) {
                                            result += notations[n4][0];
                                        } else {
                                            result += notations[n4][0] + options.delimiter_start + notations[n4][2] + options.delimiter_end;
                                        }
                                    }
                                } else {
                                    // furigana
                                    for (n5 = 0; n5 < notations.length; n5++) {
                                        if (notations[n5][1] !== 1) {
                                            result += notations[n5][0];
                                        } else {
                                            result += "<ruby>" + notations[n5][0] + "<rp>" + options.delimiter_start + "</rp><rt>" + notations[n5][2] + "</rt><rp>" + options.delimiter_end + "</rp></ruby>";
                                        }
                                    }
                                }
                                return _context2.abrupt("return", result);

                            case 64:
                                throw new Error("Unknown option.to param");

                            case 65:
                                _context2.next = 68;
                                break;

                            case 67:
                                throw new Error("No such mode...");

                            case 68:
                            case "end":
                                return _context2.stop();
                        }
                    }
                }, _callee2, this);
            }));

            function convert(_x2, _x3) {
                return _ref2.apply(this, arguments);
            }

            return convert;
        }()
    }]);

    return Kuroshiro;
}();

var Util = {
    isHiragana: _util.isHiragana,
    isKatakana: _util.isKatakana,
    isKana: _util.isKana,
    isKanji: _util.isKanji,
    isJapanese: _util.isJapanese,
    hasHiragana: _util.hasHiragana,
    hasKatakana: _util.hasKatakana,
    hasKana: _util.hasKana,
    hasKanji: _util.hasKanji,
    hasJapanese: _util.hasJapanese
};

Kuroshiro.Util = Util;

exports.default = Kuroshiro;
module.exports = exports["default"];